@@ -12,7 +12,10 @@ use JSON::PP;
 use POSIX 'floor';
 use Carp;
 
-our $VERSION = '0.1';
+our $VERSION = '0.2';
+
+# XXX need a public find_node()
+# XXX need a get_parent( $node ), and other traversal stuff
 
 # use autobox::Closure::Attributes;  # XXX hacked up local copy that permits lvalue assigns
 
@@ -73,7 +76,8 @@ Each node has this structure:
   }
 
 It may also have a C<'ch'> containing an arrayref of additional nodes.
-Since C<find> recurses for you, you may usually ignore that field.
+To make things interesting, the root node does not have a C<'ch'> of nodes under it.
+Use the C<get_children()> method to avoid dealing with this special case.
 
 The value from the C<id> field is used as the value for C<save_id>, C<node_id>, or C<parent_id>
 in other calls.
@@ -134,9 +138,13 @@ The value for C<< $wf->polling_interval >> may change in response to a request t
 Fetches the latest copy of the outline from the L<Workflowy> server, blowing away any local changes made to it that haven't yet been pushed up.
 This happens automatically on C<new>.
 
+=head2 get_children
+
+Takes a node id.  Returns an arrayref of a node's children if it has children, or false otherwise.
+
 =cut
 
-package autobox::Closure::Attributes::Methods;
+package autobox::Closure::XAttributes::Methods;
 
 use base 'autobox';
 use B;
@@ -182,11 +190,11 @@ sub AUTOLOAD :lvalue {
 
 package WWW::Workflowy;
 
-use autobox CODE => 'autobox::Closure::Attributes::Methods'; # XXX temp since we can't 'use' it because it's inline
+use autobox CODE => 'autobox::Closure::XAttributes::Methods'; # XXX temp since we can't 'use' it because it's inline
 
 sub import {
     my $class = shift;
-    $class->autobox::import(CODE => 'autobox::Closure::Attributes::Methods');
+    $class->autobox::import(CODE => 'autobox::Closure::XAttributes::Methods');
 }
 
 sub new {
@@ -199,8 +207,8 @@ sub new {
     my $outline;
     my $client_id;
     my $date_joined;
-    my $last_transaction_id;   # transaction ids are much alrger than the lastModified/lm values; eg 106551357
-    my $operations;            # edits we've made but not yet posted
+    my $last_transaction_id;   # transaction ids are much alrger than the lastModified/lm values; eg 106551357; comes from initialMostRecentOperationTransactionId then $result_json->{results}->[0]->{new_most_recent_operation_transaction_id}
+    my $operations = [];       # edits we've made but not yet posted
     my $polling_interval;      # from $outline->{initialPollingIntervalInMs} and then ->{results}->[0]->{new_polling_interval_in_ms}
     my $last_poll_time;
 
@@ -216,7 +224,7 @@ sub new {
     } elsif( $args{outline} ) {
         # testing -- pass in an outline
         $outline = delete $args{outline};
-        $last_transaction_id = $outline->{initialMostRecentOperationTransactionId};
+        $last_transaction_id = $outline->{initialMostRecentOperationTransactionId} or confess "no initialMostRecentOperationTransactionId in serialized outline";
         $date_joined = $outline->{dateJoinedTimestampInSeconds}; # XXX probably have to compute clock skew (diff between time() and this value) and use that when computing $client_timestamp
     } else {
         confess "pass guid or url";
@@ -249,20 +257,20 @@ sub new {
     
         # contains a line like this:  var mainProjectTreeInfo = { ... JSON ... };
     
-        (my $mainProjectTreeInfo) = grep $_ =~ m/var mainProjectTreeInfo /, split m/\n/, $decoded_content or die "failed to find mainProjectTreeInfo line in response"; 
-        $mainProjectTreeInfo =~ s{^\s*var mainProjectTreeInfo = }{} or die "failed to remove JS from mainProjectTreeInfo line in response";
-        $mainProjectTreeInfo =~ s{;$}{} or die;
+        (my $mainProjectTreeInfo) = grep $_ =~ m/var mainProjectTreeInfo /, split m/\n/, $decoded_content or confess "failed to find mainProjectTreeInfo line in response"; 
+        $mainProjectTreeInfo =~ s{^\s*var mainProjectTreeInfo = }{} or confess "failed to remove JS from mainProjectTreeInfo line in response";
+        $mainProjectTreeInfo =~ s{;$}{} or confess;
     
         # contains a line like this:    var clientId = "2013-02-16 21:34:52.652778";
     
-        (my $new_clientId) = grep $_ =~ m/var clientId /, split m/\n/, $decoded_content or die "failed to find clientId line in response";
-        $new_clientId =~ s{^\s*var clientId = "}{} or die "failed to remove JS from clientId line in response";
+        (my $new_clientId) = grep $_ =~ m/var clientId /, split m/\n/, $decoded_content or confess "failed to find clientId line in response";
+        $new_clientId =~ s{^\s*var clientId = "}{} or confess "failed to remove JS from clientId line in response";
         $new_clientId =~ s{";$}{};
 
         $client_id = $new_clientId;  # and nope, the new_clientId and client_id aren't generally the same; they look something like "2013-04-23 15:24:05.670771"
 
         $outline = decode_json $mainProjectTreeInfo;
-        $last_transaction_id = $outline->{initialMostRecentOperationTransactionId};
+        $last_transaction_id = $outline->{initialMostRecentOperationTransactionId} or confess "no initialMostRecentOperationTransactionId in fetch_outline";
         $date_joined = $outline->{dateJoinedTimestampInSeconds}; # XXX probably have to compute clock skew (diff between time() and this value) and use that when computing $client_timestamp
         $polling_interval = $outline->{initialPollingIntervalInMs} / 1000;
         $last_poll_time = time;
@@ -286,7 +294,7 @@ sub new {
         my %args = @_;
         my $parent_id = $args{parent_id} || $args{parent_node}->{id} or confess;
         my $new_node = $args{new_node} or confess;
-        my $priority = $args{priority};  defined $priority or confess; 
+        my $priority = $args{priority};  defined $priority or confess;
         my( $parent_node, $children ) = _find_node( $outline, $parent_id ) or confess "couldn't find node for $parent_id in edit in create_node";
         if( ! $children ) {
             if( $parent_id eq $shared_projectid ) {
@@ -331,7 +339,7 @@ sub new {
         my %args = @_;
         my $node_id = $args{node_id} or confess;
         my $parent_id = $args{parent_id} or confess;   # new parent
-        my $priority = $args{priority};  defined $priority or confess; 
+        my $priority = $args{priority};  defined $priority or confess;
 
         my $node = _find_node( $outline, $node_id ) or confess "couldn't find node for $node_id in local_move_node";
 
@@ -489,7 +497,7 @@ sub new {
 
             } elsif( $type eq 'delete' ) {
 
-                $local_delete_node->( node_id => $data->{project_id}, );
+                $local_delete_node->( node_id => $data->{projectid}, );
 
             } elsif( $type eq 'move' ) {
 
@@ -510,6 +518,8 @@ sub new {
         $r->header( 'Content-Type'     => 'application/x-www-form-urlencoded; charset=UTF-8' );
         $r->header( 'Referer'          => $workflowy_url );
 
+        $last_transaction_id or confess "no value in last_transaction_id in sync_changes";
+
         my $push_poll_data = [{
             most_recent_operation_transaction_id => $last_transaction_id,
             shared_projectid => $shared_projectid,
@@ -540,9 +550,11 @@ sub new {
         # "new_most_recent_operation_transaction_id": "106843573"
         # warn Data::Dumper::Dumper $result_json;
 
-        $last_transaction_id = $result_json->{results}->[0]->{new_most_recent_operation_transaction_id};
+        $result_json->{results}->[0]->{error} and die "workflowy.com request failed with an error: ``$result_json->{results}->[0]->{error}''; response was: $decoded_content\npush_poll_data is: " . JSON::PP->new->pretty->encode( $push_poll_data );
 
-        $polling_interval = $result_json->{results}->[0]->{new_polling_interval_in_ms} / 1000;
+        $last_transaction_id = $result_json->{results}->[0]->{new_most_recent_operation_transaction_id} or confess "no new_most_recent_operation_transaction_id in sync changes\nresponse was: $decoded_content\npush_poll_data was: " . JSON::PP->new->pretty->encode( $push_poll_data );
+
+        $polling_interval = ( $result_json->{results}->[0]->{new_polling_interval_in_ms} || 1000 )  / 1000; # XXX this was probably just undef when we ignored an error before the checking above was added
         $last_poll_time = time;
 
         #
@@ -557,6 +569,10 @@ sub new {
             $run_remote_operations->( $decoded_run_op );
         }
 
+        #
+
+        $operations = [];
+
     };
 
     #
@@ -565,8 +581,12 @@ sub new {
 
         my $action = shift;
 
+        # important symbols
+
+        $outline or confess "no outline";  # shouldn't happen
+        $shared_projectid or confess "no shared_projectid"; # shouldn't happen
+
         if( $action eq 'edit' ) {
-            $outline or confess "no outline in edit";  # shouldn't happen
             my %args = @_;
             my $save_id = delete $args{save_id} or confess "pass a save_id parameter";
             my $text = delete $args{text} or confess "pass a text parameter";
@@ -584,7 +604,6 @@ sub new {
 
             # $update_outline returns the id of the newly created node for cmd='create'
 
-            $outline or confess "no outline in create"; # shouldn't happen
             my %args = @_;
             my $parent_id = delete $args{parent_id};
             my $text = delete $args{text};
@@ -600,7 +619,6 @@ sub new {
         }
 
         if( $action eq 'delete' ) {
-            $outline or confess "no outline in delete "; # shouldn't happen
             my %args = @_;
             my $node_id = delete $args{node_id} or confess "pass a node_id parameter";
 
@@ -647,6 +665,13 @@ sub find {
     _find( $self->outline, $cb);
 }
 
+sub find_by_id {
+    # external API takes $self
+    my $self = shift;
+    my $id = shift or confess "pass id";
+    _find( $self->outline, sub { $_[0]->{id} eq $id } );
+}
+
 sub _find {
     my $outline = shift;
     my $cb = shift or confess "pass a callback";
@@ -734,6 +759,12 @@ sub _find_parent {
 
 }
 
+sub get_children {
+    my $self = shift;
+    my $node_id = shift or confess "pass a node id";
+    (undef, my $children) = _find_node( $self->outline, $node_id ) or confess;
+    return $children;
+}
 
 sub _filter_out {
     my $arr = shift or confess;
@@ -746,7 +777,6 @@ sub _filter_out {
     }
 }
 
-
 sub dump {
     my $self = shift;
 
